/*
This file is part of socklib, a cross-platform socket library.

Copyright (c) 2004 John Tsiombikas <nuclear@siggraph.org>

This library is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#ifdef __unix__
#define __USE_BSD
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/param.h>
#include <sys/select.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#else	/* assume win32 if not unix */

#include <winsock2.h>

#define MAXHOSTNAMELEN	255
#define close(s)	closesocket(s)

static int winsock_initialized;
#endif	/* __unix__ */

#include "socklib.h"

#ifdef _MSC_VER
#define __func__	__FUNCTION__
#endif	/* _MSC_VER */

static int verbose = 1;

int sl_listen(int port) {
	int s;
	struct sockaddr_in addr;
	struct hostent *host;
	char host_name[MAXHOSTNAMELEN+1];

#ifndef __unix__
	if(!winsock_initialized) {
        WSADATA wsa_data;
		WSAStartup(MAKEWORD(1, 1), &wsa_data);
		winsock_initialized = 1;
	}
#endif	/* __unix__ */
	
	if(gethostname(host_name, MAXHOSTNAMELEN+1) == -1) {
		if(verbose) perror("socklib error @ gethostname");
		return -1;
	}

	if(!(host = gethostbyname(host_name))) {
		if(verbose) perror("socklib error @ gethostbyname");
		return -1;
	}
	
	memset(&addr, 0, sizeof(struct sockaddr_in));
	addr.sin_family = host->h_addrtype;
	addr.sin_port = htons(port);
	addr.sin_addr.s_addr = INADDR_ANY;
	
	if((s = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		if(verbose) perror("socklib error @ socket");
		return -1;
	}
	
	if(bind(s, (struct sockaddr*)&addr, sizeof(struct sockaddr_in)) == -1) {
		if(verbose) perror("socklib error @ bind");
		close(s);
		return -1;
	}
	
	if(listen(s, 5) == -1) {
		if(verbose) perror("socklib error @ listen");
		close(s);
		return -1;
	}

	return s;
}


int sl_accept(int listening_socket) {
	return accept(listening_socket, 0, 0);
}

int sl_connect_tcp(char *ipaddr, const char *hostname, int port) {
	int s, must_free_ipaddr = 0;
	struct sockaddr_in addr;
	struct hostent *host;

#ifndef __unix__
	if(!winsock_initialized) {
        WSADATA wsa_data;
		WSAStartup(MAKEWORD(1, 1), &wsa_data);
		winsock_initialized = 1;
	}
#endif	/* __unix__ */
	
	if((s = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		if(verbose) perror("socklib error @ socket");
		return -1;
	}
	
	if(!ipaddr) {
		if(!hostname) return -1;
		
		ipaddr = malloc(20);
		must_free_ipaddr = 1;
		host = gethostbyname(hostname);
		
		if(!host) {
			if(verbose) perror("socklib error @ gethostbyname");
			free(ipaddr);
			return -1;
		}
		
		strcpy(ipaddr, inet_ntoa(*((struct in_addr*)host->h_addr)));
	}
		
	memset(&addr, 0, sizeof(struct sockaddr_in));
	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);
	addr.sin_addr.s_addr = inet_addr(ipaddr);

	if((connect(s, (struct sockaddr*)&addr, sizeof(struct sockaddr))) == -1) {
		if(verbose) perror("socklib error @ connect");
		if(must_free_ipaddr) free(ipaddr);
		return -1;
	}
	
	if(must_free_ipaddr) free(ipaddr);	
	return s;
}


void sl_close_socket(int s) {
	char buffer[128];
	
	shutdown(s, 1);
	while(recv(s, buffer, 128, 0));
	close(s);
}

int sl_send_data(int s, const char *buffer, int size) {
	const char *ptr = buffer;
	const char *end = buffer + size;
	
	while(ptr < end) {
		int bytes = send(s, ptr, end - ptr, 0);
		if(bytes == -1) {
			if(verbose) {
				char errstr[80];
				sprintf(errstr, "socklib error @ %s", __func__);
				perror(errstr);
			}
			return -1;
		}
		ptr += bytes;
	}
	
	return size;
}

int sl_recv_data(int s, char *buffer, int size) {
	int bytes = recv(s, buffer, size, 0);
	if(bytes == -1) {
		if(verbose) {
			char errstr[80];
			sprintf(errstr, "socklib error @ %s", __func__);
			perror(errstr);
		}
		return -1;
	}
	return bytes;
}


/* if SL_CHK_IMMED is NOT specified then:
 * if timeout is 0: block forever, else block for so many milliseconds.
 */
int sl_check_socket(int s, int flags, unsigned int timeout) {
	int res;
	fd_set rfds, wfds;
	struct timeval tv, *tvptr;
	tv.tv_sec = tv.tv_usec = 0;
	
	tvptr = (flags & SL_CHK_IMMED) ? &tv : 0;

	if(flags & SL_CHK_IMMED) {
		tvptr = &tv;
	} else {
		tvptr = timeout > 0 ? &tv : 0;
		
		tv.tv_sec = timeout / 1000;
		tv.tv_usec = (timeout - tv.tv_sec * 1000) * 1000;
	}
		

	FD_ZERO(&rfds);
	FD_ZERO(&wfds);
	
	if(flags & SL_CHK_READ) {
		FD_SET(s, &rfds);
		if((res = select(s+1, &rfds, 0, 0, tvptr)) == -1) {
			return -1;
		}
		return FD_ISSET(s, &rfds) ? 1 : 0;
	} else if(flags & SL_CHK_WRITE) {
		FD_SET(s, &wfds);
		if((res = select(s+1, 0, &wfds, 0, tvptr)) == -1) {
			return -1;
		}
		return FD_ISSET(s, &wfds) ? 1 : 0;
	}

	return -1;
}

void sl_set_verbosity(int vlevel) {
	verbose = vlevel;
}
