/*
This file is part of dynwatch, a win32 system tray applet which
updates automatically the dyndns entry of quake.gr.

Copyright (c) 2005 John Tsiombikas <nuclear@siggraph.org>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <windows.h>
#include <sensapi.h>
#include "socklib.h"
#include "watch.h"
#include "config_parser.h"
#include "locator.h"

int empty_str(const char *str);

static const char *cfg_fname = "dynwatch.conf";
char user[256], pass[256], dhost[256], hosts[256];
extern int running;

static int connected;

void init(void) {
	const struct ConfigOption *opt;

	SetParserState(PS_AssignmentSymbol, ':');
	SetParserState(PS_CommentSymbol, '#');
	
	if(LoadConfigFile(loc_get_path(cfg_fname, LOC_FILE_CONFIG)) == -1) {
		err_msg("Could not load config file, please reconfigure");
	} else {

        while((opt = GetNextOption())) {
			if(!strcmp(opt->option, "user")) {
				strncpy(user, opt->str_value, 256);
			} else if(!strcmp(opt->option, "pass")) {
				strncpy(pass, opt->str_value, 256);
			} else if(!strcmp(opt->option, "dns-host")) {
				strncpy(dhost, opt->str_value, 256);
			} else if(!strcmp(opt->option, "hosts")) {
				strncpy(hosts, opt->str_value, 256);
			} else {
				err_msg("Error parsing config file, please reconfigure");
				user[0] = pass[0] = dhost[0] = hosts[0] = 0;
			}
		}

		if(!empty_str(user) && !empty_str(pass) && !empty_str(dhost) && !empty_str(hosts)) {
			running = 1;
		}
	}

	DestroyConfigParser();

	/* force a wsastartup call */
	sl_set_verbosity(0);
	sl_connect_tcp("0.0.0.0", 0, 0);
	sl_set_verbosity(1);
}

void get_ip_address(struct in_addr *addr) {
	struct hostent *host;
	char **str_ptr;
	static char hostname[512];

	if(gethostname(hostname, 512) == -1) {
		err_msg("gethostname() failed\n");
	}

	if(!(host = gethostbyname(hostname))) {
		err_msg("gethostbyname() failed\n");
	}

	*addr = *(struct in_addr*)host->h_addr_list[0];
}

int need_update(void) {
	unsigned long flags;
	static struct in_addr addr;

	if(IsNetworkAlive(&flags)) {
		/* it was not connected before, we need to update */
		if(!connected) {
			get_ip_address(&addr);
			connected = 1;
			return 1;
		}

		/* it was connected before, but we may have missed a short
		 * disconnect. Check to see if we still have the same ip (not fullproof)
		 */
		{
			struct in_addr tmp;
			get_ip_address(&tmp);

			if(addr.S_un.S_addr != tmp.S_un.S_addr) {
				addr = tmp;
				return 1;
			}
		}

	} else {
		connected = 0;
	}
	return 0;
}


int update_dns(void) {
	int s, bytes;
	static char http_req[512];
	char *buf, *ptr;

	if(empty_str(user) || empty_str(pass) || empty_str(dhost) || empty_str(hosts)) {
		err_msg("missing configuration data, please reconfigure");
		return -1;
	}

	if((s = sl_connect_tcp(0, dhost, 80)) == -1) {
		connected = 0;
		return -1;
	}

	buf = malloc(4096);

	sprintf(buf, "upd.php?user=%s&pwd=%s&host=%s", user, pass, hosts);
	sprintf(http_req, "GET http://%s/%s HTTP/1.0\r\nUser-Agent: dynwatch/1.0\r\n\r\n", dhost, buf);

	sl_send_data(s, http_req, strlen(http_req));
	while((bytes = sl_recv_data(s, buf, 4096))) {
		if(bytes == -1) break;

		ptr = strstr(buf, "Updated");
		if(ptr) {
			free(buf);
			info_msg("dns update complete");
			sl_close_socket(s);
			return 0;
		}
	}

	free(buf);
	err_msg("dns update failed");
	sl_close_socket(s);
	return -1;
}

#define HOME_ENV	"USERPROFILE"
#define DIR_SEP		'\\'
#define MAX_PATH	512

void save_config(void) {
	FILE *fp;
	static char path[MAX_PATH];
	const char *fptr = cfg_fname;
	char *env, *pptr = path;

	env = getenv(HOME_ENV);
	if(env) {
		while(*env) *pptr++ = *env++;
		if(*(env - 1) != DIR_SEP) *pptr++ = DIR_SEP;
		while(*fptr) *pptr++ = *fptr++;
		*pptr++ = 0;
	} else {
		strcpy(path, cfg_fname);
		info_msg("warning: no USERPROFILE variable found, saving config to the current dir.");
	}

	if(!(fp = fopen(path, "w"))) {
		err_msg("could not save config file!");
		return;
	}

	fprintf(fp, "dns-host: %s\n", dhost);
	fprintf(fp, "user: %s\n", user);
	fprintf(fp, "pass: %s\n", pass);
	fprintf(fp, "hosts: %s\n", hosts);

	fclose(fp);
}